package nce.editor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import nce.ColoringEditorPlugin;
import nce.editor.prefs.ColorsPreferencePage;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.ITokenScanner;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.editors.text.TextSourceViewerConfiguration;


public class ColoringEditorConfiguration extends TextSourceViewerConfiguration {
	
	protected ColorManager colorManager;
	protected Map<String, IToken> tokenMap;

	public ColoringEditorConfiguration(IPreferenceStore preferenceStore, ColorManager colorManager) {
		
		super(preferenceStore);
		
		this.colorManager = colorManager;
		tokenMap = new HashMap<String, IToken>();
		
		
		if (ColoringEditorPlugin.getDefault() == null)
			return;
		
	}

	
	
	@Override
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		
		PresentationReconciler reconciler = new PresentationReconciler();

		for (TokenType tokenType : TokenType.values())
			createDamagerRepairer(reconciler, tokenType.name(), colorManager.getPropertyForType(tokenType));
		
		return reconciler;
	}

	@Override
	public String getConfiguredDocumentPartitioning(ISourceViewer sourceViewer) {
		// TODO Auto-generated method stub
		return super.getConfiguredDocumentPartitioning(sourceViewer);
	}

	@Override
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		List<String> result = new ArrayList<String>();
		for (TokenType tokenType : TokenType.values()) {
			result.add(tokenType.name());
		}
		return result.toArray(new String[result.size()]);
	}	
	
	public void adaptToPreferenceChange(PropertyChangeEvent event) {
		
		if (event.getProperty().contains("COLOR")) {			
			if (event.getOldValue() instanceof Boolean) {
				adaptToStyleChange(event);
			} 
			else {
				adaptToColorChange(event);
			}
		}
		
	}

	private void adaptToStyleChange(PropertyChangeEvent event) {
		
		boolean bold = false;
		
		// bold properties need to be converted to colors for the map
		String colorPlusStyle = event.getProperty();
		String colorName = colorPlusStyle.substring(0, colorPlusStyle.length() - ColorsPreferencePage.BOLD_SUFFIX.length());
		Token token = (Token) tokenMap.get(colorName);
		
		if (token == null)
			return;
		
		Object value = event.getNewValue();
		bold = ((Boolean) value).booleanValue();

		Object data = token.getData();
		if (data instanceof TextAttribute) {
			TextAttribute oldAttr = (TextAttribute) data;
			TextAttribute newAttr = new TextAttribute(oldAttr.getForeground(), oldAttr.getBackground(), bold ? SWT.BOLD : SWT.NORMAL);
			boolean isBold = (oldAttr.getStyle() == SWT.BOLD);
			if (isBold != bold) {
				token.setData(newAttr);
			}
		}
	}

	private void adaptToColorChange(PropertyChangeEvent event) {
		
		Token token = (Token) tokenMap.get(event.getProperty());
		if (token == null) {
			return;
		}
		
		RGB rgb = null;
		
		Object value = event.getNewValue();		
		if (value instanceof RGB)
			rgb = (RGB) value;
		
		else if (value instanceof String)
			rgb = StringConverter.asRGB((String) value);
		
		if (rgb != null) {
			
			String property = event.getProperty();
			Object data = token.getData();
			if (data instanceof TextAttribute) {
				TextAttribute oldAttr = (TextAttribute) data;
				TextAttribute newAttr = new TextAttribute(colorManager.getColor(property), oldAttr.getBackground(), oldAttr.getStyle());
				token.setData(newAttr);
			}
			
		}
	}

	private void createDamagerRepairer(PresentationReconciler reconciler, String contentType, String propertyName) {
		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(createScanner(propertyName));
		reconciler.setDamager(dr, contentType);
		reconciler.setRepairer(dr,  contentType);
	}

	private ITokenScanner createScanner(String propertyName) {
		RuleBasedScanner scanner = new RuleBasedScanner();
		IToken token = getToken(propertyName);
		scanner.setDefaultReturnToken(token);
		return scanner;
	}

	private IToken getToken(String propertyName) {
		
		IToken token = (IToken) tokenMap.get(propertyName);
		if (token != null)
			return token;
		
		int style = colorManager.getStyleFor(propertyName);
		Color color = colorManager.getColor(propertyName);
		TextAttribute ta = new TextAttribute(color, null, style);
		token = new Token(ta);
		tokenMap.put(propertyName, token);
		
		return token;
	}

}